@@ -262,7 +262,7 @@ let blit_zeros v ofs len =
     else begin
       blit_bits 0 bj (bpi - bj) v ofs;
       let n = ref (ofs + bpi - bj) in
-      for i = succ bi to pred ei do
+      for _i = succ bi to pred ei do
         blit_int 0 v !n;
         n := !n + bpi
       done;
@@ -278,7 +278,7 @@ let blit_ones v ofs len =
     else begin
       blit_bits max_int bj (bpi - bj) v ofs;
       let n = ref (ofs + bpi - bj) in
-      for i = succ bi to pred ei do
+      for _i = succ bi to pred ei do
         blit_int max_int v !n;
         n := !n + bpi
       done;
@@ -344,16 +344,6 @@ let foldi_right f v x =
   done;
   !r
 
-let iteri_true f v =
-  Array.iteri
-    (fun i n -> if n != 0 then begin
-       let i_bpi = i * bpi in
-       for j = 0 to bpi - 1 do
-         if n land (Array.unsafe_get bit_j j) > 0 then f (i_bpi + j)
-       done
-     end)
-    v.bits
-
 (*s Bitwise operations. It is straigthforward, since bitwise operations
     can be realized by the corresponding bitwise operations over integers.
     However, one has to take care of normalizing the result of [bwnot]
@@ -459,11 +449,11 @@ let all_ones v =
 
 let to_string v =
   let n = v.length in
-  let s = String.make n '0' in
+  let s = Buffer.create n in
   for i = 0 to n - 1 do
-    if unsafe_get v i then s.[i] <- '1'
+    Buffer.add_char s (if unsafe_get v i then '1' else '0')
   done;
-  s
+  Buffer.contents s
 
 let print fmt v = Format.pp_print_string fmt (to_string v)
 
@@ -600,10 +590,10 @@ let of_int64_us i = match Sys.word_size with
                       let hi = Int64.shift_right_logical i 62 in
                       (Int64.to_int hi) land 1 |] }
   | _ -> assert false
-let to_int64_us v = failwith "todo"
+let to_int64_us _ = failwith "todo"
 
-let of_int64_s i = failwith "todo"
-let to_int64_s v = failwith "todo"
+let of_int64_s _ = failwith "todo"
+let to_int64_s _ = failwith "todo"
 
 (* [Nativeint] *)
 let select_of f32 f64 = match Sys.word_size with
