@@ -25,10 +25,19 @@ module Int = struct
   let default = 0
 end
 
-module W = struct
-  type label = int
+(* pair with equality which ignores the second component *)
+module Pair = struct
+  type t = int * int
+  let compare (x, _) (y, _) = Int.compare x y
+  let hash (x, _) = Int.hash x
+  let equal (x, _) (y, _) = x = y
+  let default = 0, 0
+end
+
+module W(E:sig type t type label val label: t -> label end) = struct
+  type edge = E.t
   type t = int
-  let weight x = x
+  let weight = E.label
   let zero = 0
   let add = (+)
   let compare = compare
@@ -197,6 +206,69 @@ module Generic = struct
     in
     ()
 
+  (* Generic tests for imperative concrete graphs with custom equality *)
+  module Make_pair
+    (G : Sig.I with type V.label = int * int)
+    (V : sig val v: int val e: int end) =
+  struct
+
+    module O = Oper.I(G)
+    let test_mirror g =
+      if G.is_directed then begin (* TODO: remove *)
+	let g' = O.mirror g in
+	assert (G.nb_vertex g = G.nb_vertex g');
+	G.iter_edges (fun v1 v2 -> assert (G.mem_edge g' v2 v1)) g;
+	G.iter_edges (fun v1 v2 -> assert (G.mem_edge g v2 v1)) g';
+	()
+      end
+
+    let g = G.create ()
+    let () =
+      let v1 = G.V.create (1, 0) in
+      let v2 = G.V.create (2, 0) in
+      let v3 = G.V.create (2, 1) in
+      test_mirror g;
+      G.add_edge g v1 v2;
+      G.add_edge g v2 v1;
+      G.add_edge g v1 v3;
+      G.iter_vertex (fun v -> assert (snd (G.V.label v) = 0)) g;
+      test_mirror g;
+      assert (G.nb_vertex g = V.v && G.nb_edges g = V.e);
+      G.remove_vertex g v3;
+      assert (G.nb_vertex g = 1 && G.nb_edges g = 0);
+      test_mirror g;
+      G.clear g;
+      assert (G.nb_vertex g = 0 && G.nb_edges g = 0)
+
+  end
+
+  let () =
+    let module A = Make_pair
+      (Imperative.Digraph.ConcreteLabeled(Pair)(Pair))
+      (struct let v = 2 let e = 2 end)
+    in
+    let module A = Make_pair
+      (Imperative.Graph.ConcreteLabeled(Pair)(Pair))
+      (struct let v = 2 let e = 1 end)
+    in
+    let module A = Make_pair
+      (Imperative.Digraph.Concrete(Pair))
+      (struct let v = 2 let e = 2 end)
+    in
+    let module A = Make_pair
+      (Imperative.Graph.Concrete(Pair))
+      (struct let v = 2 let e = 1 end)
+    in
+    let module A = Make_pair
+      (Imperative.Digraph.ConcreteBidirectional(Pair))
+      (struct let v = 2 let e = 2 end)
+    in
+    let module A = Make_pair
+      (Imperative.Digraph.ConcreteBidirectionalLabeled(Pair)(Pair))
+      (struct let v = 2 let e = 2 end)
+    in
+    ()
+
   (* find_edge *)
 
   module Make2
@@ -251,8 +323,8 @@ end
 module Dijkstra = struct
 
   module TestDijkstra
-    (G : Sig.G with type V.label = int and type E.label = int)
-    (B : Builder.S with module G = G) =
+    (G: Sig.G with type V.label = int and type E.label = int)
+    (B: Builder.S with module G = G) =
   struct
 
     let g = B.empty ()
@@ -275,7 +347,7 @@ module Dijkstra = struct
     let g = B.add_edge_e g (G.E.create v4 20 v3)
     let g = B.add_edge_e g (G.E.create v4 60 v5)
 
-    module Dij = Path.Dijkstra(G)(W)
+    module Dij = Path.Dijkstra(G)(W(G.E))
     module Dfs = Traverse.Dfs(G)
 
     let test g i j w l =
@@ -616,6 +688,8 @@ module TestSigPack : RightSigPack = struct
   type g = t
 end
 
+include Test_clique
+
 (*
 Local Variables:
 compile-command: "make -C .. check"
